//
//  MOShadowTableView.m
//  MOShadowTableView
//
//  Created by Jan Christiansen on 9/12/12.
//  Copyright (c) 2012, Monoid - Development and Consulting - Jan Christiansen
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "MOShadowTableView.h"
#import "UIView+MOShadow.h"
#import <QuartzCore/QuartzCore.h>


const static float kDefaultShadowHeight = 20.0;

const static float kDefaultShadowAlpha = 0.5;


@implementation MOShadowTableView {

@private
    MOShadowPosition _shadowPositions;
    CAGradientLayer *_backgroundOriginShadow;
    CAGradientLayer *_backgroundBottomShadow;
    CAGradientLayer *_tableOriginShadow;
    CAGradientLayer *_tableBottomShadow;
}


#pragma mark - Initializing

- (id)initWithFrame:(CGRect)frame style:(UITableViewStyle)style {

    self = [super initWithFrame:frame style:style];
    if (self) {
        _backgroundOriginShadowHeight = kDefaultShadowHeight;
        _backgroundBottomShadowHeight = kDefaultShadowHeight;
        _tableOriginShadowHeight = kDefaultShadowHeight;
        _tableBottomShadowHeight = kDefaultShadowHeight;

        _backgroundOriginShadowAlpha = kDefaultShadowAlpha;
        _backgroundBottomShadowAlpha = kDefaultShadowAlpha;
        _tableOriginShadowAlpha = kDefaultShadowAlpha;
        _tableBottomShadowAlpha = kDefaultShadowAlpha;

        // backgroundView is used to position shadows at top and bottom of view
        super.backgroundView = [[UIView alloc] init];
        // the shadows that are attached to header and footer are not animated
        // if these have zero height
        // the following is an ugly hack that animates the shadows when the
        // last cell is deleted
        // header view is used to position a shadow at top of table
        super.tableHeaderView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 1)];
        super.tableHeaderView.backgroundColor = [UIColor whiteColor];
        // footer view is used to position a shadow at bottom of table
        super.tableFooterView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 1)];
        super.tableFooterView.backgroundColor = [UIColor whiteColor];
    }
    return self;
}


#pragma mark - Setting and Getting Properties

- (void)setBackgroundView:(UIView *)backgroundView {

    super.backgroundView = backgroundView;

    [self updateShadows];
}

- (void)setTableHeaderView:(UIView *)tableHeaderView {

    super.tableHeaderView = tableHeaderView;

    [self updateShadows];
}

- (void)setTableFooterView:(UIView *)tableFooterView {

    super.tableFooterView = tableFooterView;

    [self updateShadows];
}

- (MOShadowPosition)shadowPositions {

    return _shadowPositions;
}

- (void)setShadowPositions:(MOShadowPosition)shadowPositions {

    _shadowPositions = shadowPositions;

    [self updateShadows];
}

- (void)layoutSubviews {

    [super layoutSubviews];

    [self updateShadows];
}

// remove all shadows and add them again
- (void)updateShadows {

    // check whether top shadows are visible
    if (self.contentOffset.y < 0) {

        [_backgroundOriginShadow removeFromSuperlayer];
        if ((self.shadowPositions & MOBackgroundOriginShadow) == MOBackgroundOriginShadow) {
            _backgroundOriginShadow = [self.backgroundView
                                       addShadowWithStyle:MOViewShadowTopInside
                                       height:_backgroundOriginShadowHeight
                                       alpha:_backgroundBottomShadowAlpha];
        }

        [_tableOriginShadow removeFromSuperlayer];
        if ((self.shadowPositions & MOTableOriginShadow) == MOTableOriginShadow) {
            _tableOriginShadow = [self.tableHeaderView
                                  addShadowWithStyle:MOViewShadowTopOutside
                                  height:_tableOriginShadowHeight
                                  alpha:_tableOriginShadowAlpha];
        }
    }

    // check whether bottom shadows are visible
    if (self.contentOffset.y > self.contentSize.height - self.bounds.size.height) {
    
        [_backgroundBottomShadow removeFromSuperlayer];
        if ((self.shadowPositions & MOBackgroundBottomShadow) == MOBackgroundBottomShadow) {
            _backgroundBottomShadow = [self.backgroundView
                                       addShadowWithStyle:MOViewShadowBottomInside
                                       height:_backgroundBottomShadowHeight
                                       alpha:_backgroundBottomShadowAlpha
                                       inset:self.contentInset];
        }
        
        [_tableBottomShadow removeFromSuperlayer];
        if ((self.shadowPositions & MOTableBottomShadow) == MOTableBottomShadow) {
            _tableBottomShadow = [self.tableFooterView
                                  addShadowWithStyle:MOViewShadowBottomOutside
                                  height:_tableBottomShadowHeight
                                  alpha:_tableBottomShadowAlpha];
        }
    }
}



@end
